<?php
if (!defined('_PS_VERSION_'))
	exit;

require_once __DIR__ . '/classes/OmntInvoice.php';

class omntorderexp extends Module {
	public $supportedImports = array();
	private $_postErrors = array();
	private $omntServerUrl = "https://omnt-program.hu";
	private $omConnectionReady = false;
	private $timezone = 2;
	private $uniqTypes = array('CreateInvoice', 'ManageCustomer', 'ManageProduct', 'ManageProductPrice', 'InvalidateInvoice');
	
	protected $hooks = [
		'postUpdateOrderStatus',
		'AdminOrder'
	];
	
	public function __construct() {
		global $cookie;

		$this->name = 'omntorderexp';
		$this->tab = 'migration_tools';
		$this->version = '1.0';
		$this->author = 'OMNT Számlázó';
		$this->need_instance = 0;
		$this->ps_versions_compliancy = ['min' => '1.7.7.0', 'max' => _PS_VERSION_];

		//$this->bootstrap = true;
		parent::__construct();

		$this->displayName = $this->l('OMNT Order Exporter');
		$this->description = $this->l('This module is transmit your orders into the OMNT billing program.');

		if ((int)Configuration::get('OMNTORDEREXP_EXPSTATUS') < 1) {
			Configuration::updateValue('OMNTORDEREXP_EXPSTATUS', 2);
		}
	}
	
	private function _postValidation() {
		if (Tools::isSubmit('omntorderexpSettingsSubmit')) {
			if (!Tools::getValue('omntorderexpSettingsExpStatus')) {
				$this->_postErrors[] = $this->l('Status is required.');
			}
			
			if (!Tools::getValue('omntorderexpSettingsAccountId')) {
				$this->_postErrors[] = $this->l('Fiók (Account) azonosító megadása kötelező.');
			}
			
			if (!Tools::getValue('omntorderexpSettingsAccountAuthApiUser')) {
				$this->_postErrors[] = $this->l('API felhasználó megadása kötelező.');
			}
			
			if (!Tools::getValue('omntorderexpSettingsWebshopId')) {
				$this->_postErrors[] = $this->l('WebshopId megadása kötelző.');
			}
			
			if (!Tools::getValue('omntorderexpSettingsAccountAuthApiUserPassword') && empty(Configuration::get('OMNTORDEREXP_ACCOUNT_AUTH_API_USER_PASSWORD')) === true) {
				$this->_postErrors[] = $this->l('Jelszó megadása kötelző.');
			}
			
			if (!Tools::getValue('omntorderexpSettingsAccountAuthSignKey')) {
				$this->_postErrors[] = $this->l('Aláírókulcs megadása kötelző.');
			}
			
		}
	}

	private function _postProcess() {
		if (Tools::isSubmit('omntorderexpSettingsSubmit')) {
			Configuration::updateValue('OMNTORDEREXP_SETTINGSSERVER', Tools::getValue('omntorderexpSettingsServer'));
			
			Configuration::updateValue('OMNTORDEREXP_EXPSTATUS', Tools::getValue('omntorderexpSettingsExpStatus'));
			Configuration::updateValue('OMNTORDEREXP_ACCOUNT_ID', Tools::getValue('omntorderexpSettingsAccountId'));
			Configuration::updateValue('OMNTORDEREXP_ACCOUNT_AUTH_API_USER', Tools::getValue('omntorderexpSettingsAccountAuthApiUser'));
			if (empty(Tools::getValue('omntorderexpSettingsAccountAuthApiUserPassword')) === false) {
				Configuration::updateValue('OMNTORDEREXP_ACCOUNT_AUTH_API_USER_PASSWORD', strtoupper(hash('sha3-512', Tools::getValue('omntorderexpSettingsAccountAuthApiUserPassword'))));
			}
			Configuration::updateValue('OMNTORDEREXP_ACCOUNT_AUTH_SIGN_KEY', Tools::getValue('omntorderexpSettingsAccountAuthSignKey'));
			Configuration::updateValue('OMNTORDEREXP_WEBSHOP_ID', Tools::getValue('omntorderexpSettingsWebshopId'));
			
			Configuration::updateValue('OMNTORDEREXP_EXPINVOICESERIALHUF', Tools::getValue('omntorderexpSettingsExpInvoiceSerialHuf'));
			Configuration::updateValue('OMNTORDEREXP_EXPINVOICESERIALDEV', Tools::getValue('omntorderexpSettingsExpInvoiceSerialDev'));
			
			Configuration::updateValue('OMNTORDEREXP_SETTINGSNAMESWITCH', Tools::getValue('omntorderexpSettingsNameSwitch'));
			Configuration::updateValue('OMNTORDEREXP_SETTINGSSKU', Tools::getValue('omntorderexpSettingsSku'));
			
			Configuration::updateValue('OMNTORDEREXP_EXPINVOICEGROSSBASED', Tools::getValue('omntorderexpSettingsExpInvoiceGrossBased'));
			
			Configuration::updateValue('OMNTORDEREXP_SETTINGSZEROVATREPLACE', Tools::getValue('omntorderexpSettingsZeroVatReplace'));
			
			$omntorderexpSettingsPaymentMethod = Tools::getValue('omntorderexpSettingsPaymentMethod');
			Configuration::updateValue('OMNTORDEREXP_PAYMENT_METHOD', serialize($omntorderexpSettingsPaymentMethod));
		}
	}

	public function install() {
		if (!parent::install()) {
			return false;
		}
		
		foreach ($this->hooks as $hook) {
			if (!$this->registerHook($hook)) {
				return false;
			}
		}
		
		include(dirname(__FILE__) . '/sql/install.php');
		
		return true;
	}
	
	public function uninstall() {
		return parent::uninstall();
		
		if (!parent::uninstall()) {
			return false;
		}
		
		foreach ($this->hooks as $hook) {
			if (!$this->unregisterHook($hook)) {
				return false;
			}
		}
		
		return true;
	}
	
	private function order_state_list_in_option_list() {
		global $cookie;
		ob_start();
		$sql = 'SELECT * FROM '._DB_PREFIX_.'order_state_lang where id_lang = '.$cookie->id_lang;
		if ($results = Db::getInstance()->ExecuteS($sql)) {
			foreach ($results as $row) {
				?><option <?php if (Configuration::get('OMNTORDEREXP_EXPSTATUS') == $row['id_order_state']) { ?> selected="selected" <?php } ?> value="<?php print($row['id_order_state']); ?>"><?php print($row['name']); ?></option><?php
			}
		} else {
			?><option value="-">Nincs választható státus</option><?php
		}
		$result = ob_get_contents();
		ob_end_clean();
		return $result;
	}

	public function getContent() {
		global $cookie;
		$html = '';
		if (Tools::isSubmit('omntorderexpSettingsSubmit')) {
			$this->_postValidation();
			if (!sizeof($this->_postErrors)) {
				$this->_postProcess();
			} else {
				$html .= '<div class="alert error">';
				$html .= '<div>Mentés sikertelen!</div>';
				foreach ($this->_postErrors AS $err) {
					$html .= '<div>'. $err .'</div>';
				}
				$html .= '</div>';
			}
		}
		
		ob_start();
		
		$this->omConnectionReady = $this->omConnectionCheck();
		
		if ($this->omConnectionReady === true) {
			$omnt_payments = $this->omnt_payments();
		}
		
		?>
		<style>
			#content .alert.error {border:2px solid #ff0000; padding:12px; font-size:20px; text-align:center; color:#555555;}
		</style>
		<link rel="stylesheet" type="text/css" href="<?php print($this->_path); ?>css/css.css" />
		<div id="omntorderexp_admin_page">
			<h2><?php print($this->l('OMNT Order Exporter')); ?></h2>
			<div class="bootstrap">
				<?php
				$presta_payment_modules = $this->presta_payment_modules();
				?>
				<form action="<?php print(Tools::htmlentitiesUTF8($_SERVER['REQUEST_URI'])); ?>" method="post">
					<fieldset class="bootstrapa">
						<legend>
							<i class="material-icons mi-storage" style="vertical-align: bottom;">storage</i> <?php print($this->l('Szerver Választás')); ?>
						</legend>
						<p class="omntorderexp_export_status_p">
							<span class="switch prestashop-switch fixed-width-lg">
								<input type="radio" name="omntorderexpSettingsServer" id="omntorderexpSettingsServer_on" value="1" <?php if (Configuration::get('OMNTORDEREXP_SETTINGSSERVER') == 1) { ?> checked="checked" <?php } ?>>
								<label for="omntorderexpSettingsServer_on">Számlázó</label>
								<input type="radio" name="omntorderexpSettingsServer" id="omntorderexpSettingsServer_off" value="0" <?php if (Configuration::get('OMNTORDEREXP_SETTINGSSERVER') == 0 || empty(Configuration::get('OMNTORDEREXP_SETTINGSSERVER')) === true) { ?> checked="checked" <?php } ?>>
								<label for="omntorderexpSettingsServer_off">Teszt</label>
								<a class="slide-button btn"></a> <span style="position:absolute; margin-left:6px;">szerver</span>
							</span>
							<style>
								.bootstrap .switch input:last-of-type:checked~a {background-color: #888888; border: 1px solid #bfbfbf;
							</style>
						</p>
						<p>
							A teszt szerver használatához kérlek vedd fel velünk a kapcsolatot eMail-ben!
						</p>
					</fieldset>
					
					<fieldset>
						<legend>
							<i class="material-icons mi-format_list_bulleted" style="vertical-align: bottom;">format_list_bulleted</i> <?php print($this->l('Rendelés státusa')); ?>
						</legend>
						<p class="omntorderexp_export_status_p">
							<?php print($this->l('Bizonylat készítése ilyen státus esetén:')); ?>
							<select name="omntorderexpSettingsExpStatus"><?php print($this->order_state_list_in_option_list()); ?></select>
						</p>
						<p>
							<?php print($this->l('What is the status of orders export to the billing program?')); ?><br />
							<br />
							<?php print($this->l('Azaz ha a rendelés a fent megjelölt státusba kerül, akkor fogja a PrestaShop elküldeni a rendelés adatait az OMNT számlázónak, ahol létrejön a megfelelő bizonylat (pl.: számla, díjbekérő).')); ?>
						</p>
					</fieldset>
					
					<fieldset>
						<legend>
							<i class="material-icons mi-account_circle" style="vertical-align: bottom;">account_circle</i> <?php print($this->l('Authentication')); ?>
						</legend>
						<table>
							<tr>
								<td><?php print($this->l('AccountId:')); ?></td>
								<td><input class="text" name="omntorderexpSettingsAccountId" value="<?php print(Configuration::get('OMNTORDEREXP_ACCOUNT_ID')); ?>" type="text" /></td>
							</tr>
							<tr>
								<td><?php print($this->l('ApiUser:')); ?></td>
								<td><input class="text" name="omntorderexpSettingsAccountAuthApiUser" value="<?php print(Configuration::get('OMNTORDEREXP_ACCOUNT_AUTH_API_USER')); ?>" type="text" /></td>
							</tr>
							<tr>
								<td><?php print($this->l('ApiUserPassword:')); ?></td>
								<td><input class="text" name="omntorderexpSettingsAccountAuthApiUserPassword" type="password" /></td>
							</tr>
							<tr>
								<td><?php print($this->l('SignKey:')); ?></td>
								<td><input class="text" name="omntorderexpSettingsAccountAuthSignKey" value="<?php print(Configuration::get('OMNTORDEREXP_ACCOUNT_AUTH_SIGN_KEY')); ?>" type="text" /></td>
							</tr>
							<tr>
								<td>Teszt kapcsolódás</td>
								<td>
									<?php
									if ($this->omConnectionReady === true) {
										?><div style="margin:2px; color:#209020; font-weight:700;">Kapcsolat rendben</div><?php
									} else {
										?><div style="margin:2px; color:#ff0000;">Sikertelen kapcsolódás</div><?php
									}
									?>
								</td>
							</tr>
							<tr>
								<td><?php print($this->l('WebshopId:')); ?></td>
								<td>ps-<input class="text" name="omntorderexpSettingsWebshopId" id="omntorderexpSettingsWebshopId" value="<?php print(Configuration::get('OMNTORDEREXP_WEBSHOP_ID')); ?>" type="text" /></td>
							</tr>
						</table>
						<style>
							input#omntorderexpSettingsWebshopId {display:inline-block;}
						</style>
						<p>
							<?php print($this->l('Az azonosításhoz szükséges adatok az OMNT számlázó beállításai között találhatóak: "Beállítások" -> "Kapcsolatok" menüpont. Az OMNT számlázóban bármikor újragenerálhatóak a kapcsolathoz tartozó adatok (pl.: API felhasználó, jelszó, stb), de ilyenkor sose felejtsd el itt, a PrestaShop-ban is frissíteni azt!')); ?><br />
							<br />
							<?php print($this->l('A WebshopId (webshop azonosító) megadása kötelező. Ide bármit beírhatsz, amivel be tudod azonosítani webáruházad.')); ?>
						</p>
					</fieldset>
					
					<fieldset>
						<legend>
							<i class="material-icons mi-content_copy" style="vertical-align: bottom;">content_copy</i> <?php print($this->l('Automata bizonylat készítés (számla, díjbekérő, stb)')); ?>
						</legend>
						<table>
							<?php
							$serialTypes = array(
								1 => 'Forint számlatömb',
								2 => 'Forint díjbekérőtömb',
								3 => 'Deviza számlatömb',
								4 => 'Deviza díjbekérőtömb',
								6 => 'Forint web megrendelés tömb',
								7 => 'Deviza web megrendelés tömb'
							);
							
							$serialTypeTypes = array(
								1 => 'out',
								2 => 'proforma',
								3 => 'out',
								4 => 'proforma',
								6 => 'weborder',
								7 => 'weborder'
							);
							?>
							<tr>
								<td>
									<?php print($this->l('HUF deviza esetén:')); ?>
								</td>
								<td>
									<select name="omntorderexpSettingsExpInvoiceSerialHuf">
										<option value="0">Ne legyen kiállítva bizonylat</option>
										<?php
										foreach ($this->getInvoiceSerials() as $row) {
											?><option <?php if (Configuration::get('OMNTORDEREXP_EXPINVOICESERIALHUF') == $row['id'] . '-' . $serialTypeTypes[$row['type']]) { ?> selected="selected" <?php } ?> value="<?php print($row['id'] . '-' . $serialTypeTypes[$row['type']]); ?>"><?php print(str_replace(array("[prefix]", "[no]", "[year]"), array($row['prefix'], $row['next_no'], $row['year']), $row['pattern']) . ' (' . $serialTypes[$row['type']] . ')'); ?></option><?php
										}
										?>
									</select>
								</td>
							</tr>
							<tr>
								<td>
									<?php print($this->l('Más deviza esetén')); ?>
								</td>
								<td>
									<select name="omntorderexpSettingsExpInvoiceSerialDev">
										<option value="0">Ne legyen kiállítva bizonylat</option>
										<?php
										foreach ($this->getInvoiceSerials(true) as $row) {
											?><option <?php if (Configuration::get('OMNTORDEREXP_EXPINVOICESERIALDEV') == $row['id'] . '-' . $serialTypeTypes[$row['type']]) { ?> selected="selected" <?php } ?> value="<?php print($row['id'] . '-' . $serialTypeTypes[$row['type']]); ?>"><?php print(str_replace(array("[prefix]", "[no]", "[year]"), array($row['prefix'], $row['next_no'], $row['year']), $row['pattern']) . ' (' . $serialTypes[$row['type']] . ')'); ?></option><?php
										}
										?>
									</select>
								</td>
							</tr>
							<tr>
								<td>
									<?php print($this->l('A számlázás alapja')); ?>
								</td>
								<td>
									<select name="omntorderexpSettingsExpInvoiceGrossBased">
										<option <?php if (Configuration::get('OMNTORDEREXP_EXPINVOICEGROSSBASED') == 0) { ?> selected="selected" <?php } ?> value="0">Nettó alapú</option>
										<option <?php if (Configuration::get('OMNTORDEREXP_EXPINVOICEGROSSBASED') == 1) { ?> selected="selected" <?php } ?> value="1">Bruttó alapú</option>
									</select>
								</td>
							</tr>
						</table>
						<p>
							<?php print($this->l('A bizonylat (számla, díjbekérő, stb) a választott bizonylat tömb beállításai alapján lesz létrehozva (pl.: számla kiküldése automatikusan, megjegyzés, számla nyelve, stb). Ezeket ellenőrizd a számlázóban!')); ?><br />
						</p>
					</fieldset>
					
					<fieldset>
						<legend>
							<i class="material-icons mi-settings" style="vertical-align: bottom;">settings</i> <?php print($this->l('Beállítások')); ?>
						</legend>
						<table>
							<tr>
								<td>
									<?php print($this->l('Keresztnév és vezetéknév felcserélése')); ?>
								</td>
								<td>
									<span class="switch prestashop-switch fixed-width-lg">
										<input type="radio" name="omntorderexpSettingsNameSwitch" id="omntorderexpSettingsNameSwitch_on" value="1" <?php if (Configuration::get('OMNTORDEREXP_SETTINGSNAMESWITCH') == 1) { ?> checked="checked" <?php } ?>>
										<label for="omntorderexpSettingsNameSwitch_on">Igen</label>
										<input type="radio" name="omntorderexpSettingsNameSwitch" id="omntorderexpSettingsNameSwitch_off" value="0" <?php if (Configuration::get('OMNTORDEREXP_SETTINGSNAMESWITCH') == 0 || empty(Configuration::get('OMNTORDEREXP_SETTINGSNAMESWITCH')) === true) { ?> checked="checked" <?php } ?>>
										<label for="omntorderexpSettingsNameSwitch_off">Nem</label>
										<a class="slide-button btn"></a>
									</span>
								</td>
							</tr>
							<tr>
								<td>
									<?php print($this->l('Cikkszámok megjelenítése a számlán')); ?>
								</td>
								<td>
									<span class="switch prestashop-switch fixed-width-lg">
										<input type="radio" name="omntorderexpSettingsSku" id="omntorderexpSettingsSku_on" value="1" <?php if (Configuration::get('OMNTORDEREXP_SETTINGSSKU') == 1) { ?> checked="checked" <?php } ?>>
										<label for="omntorderexpSettingsSku_on">Igen</label>
										<input type="radio" name="omntorderexpSettingsSku" id="omntorderexpSettingsSku_off" value="0" <?php if (Configuration::get('OMNTORDEREXP_SETTINGSSKU') == 0 || empty(Configuration::get('OMNTORDEREXP_SETTINGSSKU')) === true) { ?> checked="checked" <?php } ?>>
										<label for="omntorderexpSettingsSku_off">Nem</label>
										<a class="slide-button btn"></a>
									</span>
								</td>
							</tr>
							<tr>
								<td>
									<?php print($this->l('ÁFA felülírás, ha 0%')); ?>
								</td>
								<td>
									<select name="omntorderexpSettingsZeroVatReplace">
										<?php
										$omntVatRates = array('-' => $this->l('Ne legyen számlázva')) + $this->omntVatRates();
										$omntorderexpSettingsZeroVatReplace = Configuration::get('OMNTORDEREXP_SETTINGSZEROVATREPLACE');
										foreach ($omntVatRates as $oneOmntVatRateKey => $oneOmntVatRate) {
											?><option <?php if ($omntorderexpSettingsZeroVatReplace === $oneOmntVatRateKey) { ?> selected="selected" <?php } ?> value="<?php print($oneOmntVatRateKey); ?>"><?php print($oneOmntVatRate); ?></option><?php
										}
										?>
									</select>
								</td>
							</tr>
						</table>
					</fieldset>
					<br />
					<fieldset>
						<legend>
							<i class="material-icons mi-payment" style="vertical-align: bottom;">payment</i> <?php print($this->l('Payment methods')); ?>
						</legend>
						<?php
						if (is_array($omnt_payments) === true and is_array($presta_payment_modules) === true) {
							$omntorderexpSettingsPaymentMethod = unserialize(Configuration::get('OMNTORDEREXP_PAYMENT_METHOD'));
							?>
							<table>
								<tr>
									<th><?php print($this->l('PrestaShop payment methods')); ?></th>
									<th><?php print($this->l('OMNT payment methods')); ?></th>
								</tr>
								<?php
								foreach ($presta_payment_modules as $one_module) {
									?>
									<tr>
										<td>
											<?php printf($this->l('%s esetén: '), '<span title="'.$one_module->description.'">"'.$one_module->displayName.'"</span>'); ?>
										</td>
										<td>
											<select name="omntorderexpSettingsPaymentMethod[<?php print($one_module->name); ?>]">
												<option value="-#-"><?php print($this->l('Ne legyen kiállítva bizonylat')); ?></option>
												<?php
												foreach ($omnt_payments as $one_omnt_payment) {
													?><option <?php if ($omntorderexpSettingsPaymentMethod["$one_module->name"] === $one_omnt_payment["id"]) { ?> selected="selected" <?php } ?> value="<?php print($one_omnt_payment["id"]); ?>"><?php print($one_omnt_payment["name"]); ?> (<?php printf($this->l('Deadline of payment: %s'), $one_omnt_payment["deadline_days"]); ?>)</option><?php
												}
												?>
											</select>
										</td>
									</tr>
									<?php
								}
								?>
							</table>
							<p>
								<?php print($this->l('Itt lehet összekapcsolni a PrestaShop fizetési módjait az OMNT számlázó fizetési módjaival. Azaz, hogy egy PrestaShop-beli fizetési mód az OMNT számlázóba miként kerüljön be, minek feleljen meg.')); ?><br />
								<br />
								<?php print($this->l('Ha valamihez nem választasz OMNT-beli fizetési módot, akkor ha a vevő azt választja fizetési módjának, akkor a rendszer nem fogja elküldeni a megrendelés adatait az OMNT számlázónak, hiszen nem tudná ott semmilyen fizetési módhoz párosítani.')); ?><br />
								<br />
								<?php print($this->l('Fontos, hogyha az OMNT számlázóban felveszel egy új fizetési módot, vagy módosítasz/törölsz egy meglévőt, mindenképpen ellenőrizzd itt a PrestaShop-ban, hogy nem-e kell korrigálni ennek megfelelően az itteni beállításokat! Illetve fordítva is, ha a PrestaShop-ba kerül/módosul egy fizetési mód, akkor mindenképpen ellenőrizd az itt beállítható párosításokat!')); ?>
							</p>
							<?php
						} else {
							if (is_array($omnt_payments) === false) {
								print($this->l('Error! No OMNT payment(s)...'));?><br /><?php
								print($this->l('Előfordulhat, hogy hibás valmelyik azonosító, vagy jelszó!'));?><br /><?php
							}
							if (is_array($presta_payment_modules) === false) {
								print($this->l('Error! No PrestaShop payment module(s)...'));?><br /><?php
								print($this->l('Előfordulhat, hogy hibás valmelyik azonosító, vagy jelszó!'));?><br /><?php
							}
						}
						?>
					</fieldset>
					<fieldset>
						<button type="submit" value="1" id="configuration_form_submit_btn_4" name="omntorderexpSettingsSubmit" class="btn btn-default pull-right">
							<i class="process-icon-save"></i> <?php print($this->l('Save')); ?>
						</button>
					</fieldset>
					<fieldset>
						<legend>
							<i class="material-icons mi-content_paste" style="vertical-align: bottom;">content_paste</i> <?php print($this->l('Licence')); ?>
						</legend>
						<div style="width: 420px; margin-top:12px; float:left; text-align:right;"><a href="https://creativecommons.org/licenses/by-nc-nd/3.0/legalcode"><img src="<?php print($this->_path); ?>img/creative-commons-by-nc-nd.png" alt="" style="width:88px; height:31px;" /></a></div>
						<p>
							Nevezd meg! - Ne add el! - Ne változtasd!
							<br /><a class="omntorderexp_link" href="https://creativecommons.org/licenses/by-nc-nd/3.0/legalcode">CC BY-NC-ND</a>
							<br /><a class="omntorderexp_link" href="https://omnt-szamla.hu">omnt-szamla.hu</a>
						</p>
					</fieldset>
				</form>
			</div>
		</div>
		<?php
		$html .= ob_get_contents();
		ob_end_clean();
		return $html;
	}
	
	private function getInvoiceSerials($dev = false) {
		$invoiceSerials = array();
		$xml = $this->createXml("GetInvoiceSerialList", "<Request>
			<Page>1</Page>
			<Limit>100</Limit>
			<Order>id</Order>
			<OrderDirection>ASC</OrderDirection>
			<QueryParams>
				<QueryParam>
					<QueryBase>year</QueryBase>
					<QueryOperator>EQ</QueryOperator>
					<QueryValue>" . date("Y", time()) . "</QueryValue>
				</QueryParam>
				<QueryParam>
					<QueryBase>type</QueryBase>
					<QueryOperator>GT</QueryOperator>
					<QueryValue>" . (($dev === true) ? 2 : 0) . "</QueryValue>
				</QueryParam>
				<QueryParam>
					<QueryBase>type</QueryBase>
					<QueryOperator>LT</QueryOperator>
					<QueryValue>" . (($dev === true) ? 5 : 3) . "</QueryValue>
				</QueryParam>
			</QueryParams>
		</Request>");
		
		$result = $this->curlCall("GetInvoiceSerialList", $xml);
		$resultArray = json_decode(json_encode(simplexml_load_string(mb_substr($result, mb_strpos($result, '<?xml')), "SimpleXMLElement", LIBXML_NOCDATA)), true);
		
		$invoiceSerials = $this->invoiceSerialToArray($invoiceSerials, $resultArray);
		
		$xml = $this->createXml("GetInvoiceSerialList", "<Request>
			<Page>1</Page>
			<Limit>100</Limit>
			<Order>id</Order>
			<OrderDirection>ASC</OrderDirection>
			<QueryParams>
				<QueryParam>
					<QueryBase>year</QueryBase>
					<QueryOperator>EQ</QueryOperator>
					<QueryValue>" . date("Y", time()) . "</QueryValue>
				</QueryParam>
				<QueryParam>
					<QueryBase>type</QueryBase>
					<QueryOperator>EQ</QueryOperator>
					<QueryValue>" . (($dev === true) ? 7 : 6) . "</QueryValue>
				</QueryParam>
			</QueryParams>
		</Request>");
		
		$result = $this->curlCall("GetInvoiceSerialList", $xml);
		$resultArray = json_decode(json_encode(simplexml_load_string(mb_substr($result, mb_strpos($result, '<?xml')), "SimpleXMLElement", LIBXML_NOCDATA)), true);
		
		$invoiceSerials = $this->invoiceSerialToArray($invoiceSerials, $resultArray);
		
		return $invoiceSerials;
	}
	
	private function invoiceSerialToArray($invoiceSerials, $resultArray) {
		if (isset($resultArray['Result']['list']['InvoiceSerial']) === true) {
			if ($resultArray['Result']['count'] == 1) {
				$invoiceSerials[] = $resultArray['Result']['list']['InvoiceSerial'];
			} else {
				foreach($resultArray['Result']['list']['InvoiceSerial'] as $oneSerial) {
					$invoiceSerials[] = $oneSerial;
				}
			}
		}
		return $invoiceSerials;
	}
	
	private function omConnectionCheck() {
		$return = false;
		$xml = $this->createXml("GetPaymentMethodList", "<Request>
			<Page>1</Page>
			<Limit>1</Limit>
			<Order>id</Order>
			<OrderDirection>ASC</OrderDirection>
		</Request>");
		$result = $this->curlCall("GetPaymentMethodList", $xml);
		$resultArray = json_decode(json_encode(simplexml_load_string(mb_substr($result, mb_strpos($result, '<?xml')), "SimpleXMLElement", LIBXML_NOCDATA)), true);
		
		if (isset($resultArray['Result']['list']['PaymentMethod']) === true) {
			$return = true;
		}
		return $return;
	}
	
	private function curlCall($type, $xml) {
		$server = 'teszt.omnt-program.hu';
		$authServer = Configuration::get('OMNTORDEREXP_SETTINGSSERVER');
		if (empty($authServer) === false && $authServer == 1) {
			$server = 'omnt-program.hu';
		}
		
		$url = 'https://' . $server . '/public/apis/requestProcessing/' . $type;
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_VERBOSE, 1);
		curl_setopt($ch, CURLOPT_HEADER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/xml', 'Accept: application/xml'));
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
		$result = curl_exec($ch);
		if (!curl_errno($ch)) {
			$info = curl_getinfo($ch);
			if ($info['http_code'] != 200) {
				$result = '';
			}
		}
		curl_close($ch);
		return $result;
	}
	
	private function presta_payment_modules() {
		$payment_modules = false;
		$modules = Module::getModulesOnDisk(true);
		foreach ($modules as $module) {
			if ($module->tab == 'payments_gateways' and $module->active == 1) {
				$payment_modules[] = $module;
			}
		}
		return $payment_modules;
	}
	
	private static function createDataCheck($requestId, $timeNow, $request) {
		$timeStamp = gmdate("Y-m-d\TH:i:s\.000\Z", $timeNow /*+ 3600*(self::$timezone + date("I"))*/);
		$timeStampToSign = new DateTime($timeStamp);
		return "<DataCheck>
			<signature>" . strtoupper(hash('sha3-512', $requestId . $timeStampToSign->format('YmdHis') . Configuration::get('OMNTORDEREXP_ACCOUNT_AUTH_SIGN_KEY') . base64_encode($request))) . "</signature>
		</DataCheck>";
	}
	
	private function createXml($type, $request) {
		$timeNow = time();
		$requestId = '#' . $timeNow;
		$dataCheck = "";
		if (in_array($type, $this->uniqTypes) === true) {
			$requestId = $timeNow;
			$dataCheck = $this->createDataCheck($requestId, $timeNow, $request);
		}
		
		return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>
		<" . $type . ">
			<Header>
				<requestId>" . $requestId . "</requestId>
				<timestamp>" . gmdate("Y-m-d\TH:i:s\.000\Z", $timeNow /*+ 3600*(self::$timezone + date("I"))*/) . "</timestamp>
				<api_ver>2.0</api_ver>
				<response_content_format>xml</response_content_format>
			</Header>
			<AccountAuth>
				<account_id>" . Configuration::get('OMNTORDEREXP_ACCOUNT_ID') . "</account_id>
				<api_user>" . Configuration::get('OMNTORDEREXP_ACCOUNT_AUTH_API_USER') . "</api_user>
				<password>" . Configuration::get('OMNTORDEREXP_ACCOUNT_AUTH_API_USER_PASSWORD') . "</password>
				<webshop_id>ps-" . Configuration::get('OMNTORDEREXP_WEBSHOP_ID') . "</webshop_id>
			</AccountAuth>
			" . $dataCheck . "
			" . $request . "
		</" . $type . ">";
	}
	
	private function omnt_payments() {
		$payments = array();
		$xml = $this->createXml("GetPaymentMethodList", "<Request>
			<Page>1</Page>
			<Limit>100</Limit>
			<Order>id</Order>
			<OrderDirection>ASC</OrderDirection>
		</Request>");
		$result = $this->curlCall("GetPaymentMethodList", $xml);
		$resultArray = json_decode(json_encode(simplexml_load_string(mb_substr($result, mb_strpos($result, '<?xml')), "SimpleXMLElement", LIBXML_NOCDATA)), true);
		
		if (isset($resultArray['Result']['list']['PaymentMethod']) === true) {
			foreach($resultArray['Result']['list']['PaymentMethod'] as $onePayment) {
				$payments[] = array('id' => $onePayment['id'],
					'name' => $onePayment['name'],
					'deadline_days' => $onePayment['deadline_days']
				);
			}
		}
		return $payments;
	}
	
	private function omntVatRates() {
		$vatRates = array();
		$xml = $this->createXml("GetVatRateList", "<Request>
			<Page>1</Page>
			<Limit>50</Limit>
			<Order>rate</Order>
			<OrderDirection>DESC</OrderDirection>
		</Request>");
		$result = $this->curlCall("GetVatRateList", $xml);
		$resultArray = json_decode(json_encode(simplexml_load_string(mb_substr($result, mb_strpos($result, '<?xml')), "SimpleXMLElement", LIBXML_NOCDATA)), true);
		
		if (isset($resultArray['Result']['list']['VatRate']) === true) {
			foreach($resultArray['Result']['list']['VatRate'] as $oneVatRate) {
				$vatRates[str_replace('%', '', $oneVatRate['short_name'])] = $oneVatRate['short_name'] . ((empty($oneVatRate['long_name']) === false) ? ' (' . $oneVatRate['long_name'] . ')' : '');
			}
		}
		return $vatRates;
	}
	
	private static function applyVatRule($tax, $isShipping = false) {
		$overrideWhenZero = Configuration::get('OMNTORDEREXP_SETTINGSZEROVATREPLACE');
		
		if ($overrideWhenZero != '-' && in_array(trim($tax), array(0, '0', '0%', '0 %')) === true) {
			return $overrideWhenZero;
		}
		
		return (int)$tax;
	}
	
	private function checkTaxNumber($taxNumber) {
		$pattern = '/^(\d{7})(\d)\-([1-5])\-(0[2-9]|[13][0-9]|2[02-9]|4[0-4]|51)$/';
		preg_match($pattern, $taxNumber, $matches);
		//$matches = taxNumber.match(pattern);
		if ($matches) {
			// Szorzók az adószám törzsszám része (első 8 jegy) ellenőrző számjegyének
			// meghatározásához (a 8. jegy az első hét jegy alapján). Az adott szorzóval, a
			// törzsszám megfelelő számjegyét (elölről indulva) megszorozzuk, majd ezen
			// szorzatok összegét képezzük.
			$mul = [9, 7, 3, 1, 9, 7, 3];
			// Az adószám törzsszám részének első 7 jegye.
			$base = $matches[1]/*.split("")*/;
			// Ellenőrző számjegy (az adószám 8. jegye).
			$check = /*parseInt*/(int)($matches[2]);
			// Ellenőrző összeg meghatározása.
			$sum = 0;
			for ($i = 0; $i < 7; $i++) {$sum += /*parseInt*/(int)($base[$i]) * $mul[$i]; }
			// Az ellenőrző összeg utolsó jegyének meghatározása (= 10-zel való osztás maradéka).
			$last = $sum % 10;
			// Ha a kiszámított ellenőrző számjegy nem nulla, akkor 10-ből kivonjuk.
			if ($last > 0) {$last = 10 - $last;}
			// A kiszámított és kapott ellenőrző számjegyek egyezősége alapján
			// lesz helyes vagy helytelen az adószám.
			return $last === $check;
		}
		return false;
	}
	
	private function CreateInvoice($params) {
		
		$omInvoice = OmntInvoice::getOrCreateByIdOrder($params["id_order"]);
        if ($omInvoice->hasInvoice()) {
			Configuration::updateValue('OMNTORDEREXP_ORDER_MESSAGE', 'A rendelés már készül bizonylat (számla, díjbekérő), minek sorszáma: ' . $omInvoice->getSerialNumber() . '. Ezért nem generált a rendszer egy újat automatikusan. Ha mégis szükség lenne rá, kérlek lépj be a számlázóba és hozd létre manuálisan.');
            return;
        }
		
		$order = new Order($params["id_order"]);
		$currency = new Currency($order->id_currency);
		$customer = new Customer($order->id_customer);
		$addressInvoice = new Address(intval($order->id_address_invoice));
		$addressDelivery = new Address(intval($order->id_address_delivery));
		$products = $order->getProducts();
		
		if ($addressInvoice->vat_number) {
			$taxCode = $addressInvoice->vat_number;
		} elseif ($addressInvoice->dni) { // some plugins use dni field instead of vat_number
			$taxCode = $addressInvoice->dni;
		} else {
			$taxCode = '';
		}
		
		$customerType = null;
		if (empty($taxCode) === true) {
			$customerType = "PRIVATE_PERSON";
		} elseif ($this->checkTaxNumber($taxCode) === true && empty($addressInvoice->company) === false) {
			$customerType = 'DOMESTIC';
		}
		
		if ($customerType === null) {
			Configuration::updateValue('OMNTORDEREXP_ORDER_MESSAGE', 'Hibás vagy külföldi adószám miatt nem lett kiállítva a számla. Javítsd az adószámot és próbálkozz újra, vagy állítsd ki kézzel a számlát.');
		}
		
		$invoiceSerial = explode('-', Configuration::get('OMNTORDEREXP_EXPINVOICESERIALHUF'));
		if ($currency->iso_code != 'HUF') {
			$invoiceSerial = explode('-', Configuration::get('OMNTORDEREXP_EXPINVOICESERIALDEV'));
		}
		
		if (isset($invoiceSerial[0]) === true and $invoiceSerial[0] > 0 && $customerType !== null) {
			$grossBased = (int)Configuration::get('OMNTORDEREXP_EXPINVOICEGROSSBASED');
			$productsXml = "";
			$skuOnInvoice = Configuration::get('OMNTORDEREXP_SETTINGSSKU');
			
			foreach ($products as $one_product) {
				$productsXml .= "<oneLine><line>
					<name>" . $one_product['product_name'] . "</name>
					" . (($skuOnInvoice == 1 && empty($one_product['reference']) === false) ? "<serial>" .   $one_product['reference'] . "</serial>" : "") . "
					<tax>" . $this->applyVatRule($one_product['tax_rate']) . "</tax>
					" . (($grossBased == 1) ? "<gross_price>" . floatval($one_product['unit_price_tax_incl']) . "</gross_price>" : "<net_price>" . floatval($one_product['unit_price_tax_excl']) . "</net_price>") . "
					<qty>" . $one_product['product_quantity'] . "</qty>
					<unit>" . ((empty($one_product['unity']) === false) ? $one_product['unity'] : 'db') . "</unit>
				</line></oneLine>";
			}
			
			$cartRules = $order->getCartRules();
			foreach ($cartRules as $cartRule) {
				if (!$cartRule['value']) {
					continue;
				}
				
				$productsXml .= "<oneLine><line>
					<name>" . $this->l('Kedvezmény') . ': ' . $cartRule['name'] . "</name>
					<tax>" . $this->applyVatRule(floatval(($cartRule['value'] / $cartRule['value_tax_excl']) - 1)) . "</tax>
					" . (($grossBased == 1) ? "<gross_price>" . floatval(-1.0 * $cartRule['value']) . "</gross_price>" : "<net_price>" . floatval(-1.0 * $cartRule['value_tax_excl']) . "</net_price>") . "
					<qty>1</qty>
					<unit>db</unit>
				</line></oneLine>";
			}
			
			if ($order->module == 'codpro' && ($codpro = Module::getInstanceByName('codpro'))) {
				$cp = $codpro->getCodFeeByOrderID($params["id_order"]);
				$ct = $codpro->getChargeTaxValue();
				
				$productsXml .= "<oneLine><line>
					<name>" . $this->l('Utánvét') . "</name>
					<tax>" . $this->applyVatRule($ct ? (int)round($ct / $cp['codfee'] * 100) : 0) . "</tax>
					" . (($grossBased == 1) ? "<gross_price>" . (float)$cp['codfee'] . "</gross_price>" : "<net_price>" . (float)$cp['codfee'] . "</net_price>") . "
					<qty>1</qty>
					<unit>db</unit>
				</line></oneLine>";
			}
			
			$shipping = $order->getShipping();
			foreach ($shipping as $ship) {
				$shippingPrice = floatval($ship['shipping_cost_tax_excl']);
				
				if ($shippingPrice === 0.0) {
					break;
				}
				
				$productsXml .= "<oneLine><line>
					<name>" . $ship['carrier_name'] . "</name>
					<tax>" . $this->applyVatRule($order->carrier_tax_rate) . "</tax>
					" . (($grossBased == 1) ? "<gross_price>" . floatval($ship['shipping_cost_tax_incl']) . "</gross_price>" : "<net_price>" . $shippingPrice . "</net_price>") . "
					<qty>1</qty>
					<unit>db</unit>
				</line></oneLine>";
			}
			
			$omntorderexpSettingsPaymentMethod = unserialize(Configuration::get('OMNTORDEREXP_PAYMENT_METHOD'));
			
			$customerName = $addressInvoice->firstname . ' ' . $addressInvoice->lastname;
			if (Configuration::get('OMNTORDEREXP_SETTINGSNAMESWITCH') == '1') {
				$customerName = $addressInvoice->lastname . ' ' . $addressInvoice->firstname;
			}
			
			if (empty($taxCode) === false) {
				$taxCode = explode('-', $taxCode);
			}
			
			$xml = $this->createXml("CreateInvoice", "<Request>
				<Head>
					<invoice_serial_id>" . $invoiceSerial[0] . "</invoice_serial_id>
					<type>" . $invoiceSerial[1] . "</type>
					<!--<xs:element name=bankAccount type=BankAccountType minOccurs=0>-->
					<currency>" . $currency->iso_code . "</currency>
					" . (($currency->iso_code == 'HUF') ? '' : '<exchange_rate>' . $currency->conversion_rate . '</exchange_rate>') . "
					<payment_method_id>" . $omntorderexpSettingsPaymentMethod[$order->module] . "</payment_method_id>
					<document_date>" . date("Y-m-d", time()) . "</document_date>
					<!--<performance_date></performance_date>-->
					<!--<deadline_date></deadline_date>-->
					<advance>0</advance>
					<gross_based>" . Configuration::get('OMNTORDEREXP_EXPINVOICEGROSSBASED') . "</gross_based>
					<!--
					<header_type>0</header_type>
					<language>0</language>
					<show_original>0</show_original>
					<maker_signature>0</maker_signature>
					<recipient_signature>0</recipient_signature>
					<maker_name>0</maker_name>
					<cash_accounting>0</cash_accounting>
					<payed>0</payed>
					<online_invoice_to_send>1</online_invoice_to_send>
					<notify>0</notify>
					<copy_count>2</copy_count>
					<comment> </comment>
					<bottom_comment>0</bottom_comment>
					<comment_monospaced>0</comment_monospaced>
					-->
				</Head>
				<Customer>
					<email>" . $customer->email . "</email>
					<name>" . (($customerType = 'DOMESTIC') ? $addressInvoice->company : $customerName) . "</name>
					<type>" . $customerType . "</type>
					<billing_simple_address>
						<country_code>" . @$addressInvoice->country_iso . "</country_code>
						<country>" . @$addressInvoice->country . "</country>
						<zip>" . $addressInvoice->postcode . "</zip>
						<city>" . $addressInvoice->city . "</city>
						<address>" . $addressInvoice->address1 . ' ' . $addressInvoice->address2 . "</address>
					</billing_simple_address>
					" . (($customerType === 'PRIVATE_PERSON') ? "" : "<tax_number><tax_number_0>" . $taxCode[0] . "</tax_number_0><tax_number_1>" . $taxCode[1] . "</tax_number_1><tax_number_2>" . $taxCode[2] . "</tax_number_2></tax_number>") . "
					<!--<eu_tax_number></eu_tax_number>-->
				</Customer>
				<Lines>
					" . $productsXml . "
				</Lines>
			</Request>");
			
			$result = $this->curlCall("CreateInvoice/" . ucfirst($invoiceSerial[1]), $xml);
			$resultArray = json_decode(json_encode(simplexml_load_string(mb_substr($result, mb_strpos($result, '<?xml')), "SimpleXMLElement", LIBXML_NOCDATA)), true);
			
			if (empty($resultArray['Result']['Invoice']['id']) === false) {
				Configuration::updateValue('OMNTORDEREXP_ORDER_MESSAGE', 'A rendeléshez tartozó bizonylat sikeresen létre lett hozva az OMNT Számlázó rendszerében ' . $resultArray['Result']['Invoice']['serial_number'] . ' sorszámmal.');
				$omInvoice->omnt_id = $resultArray['Result']['Invoice']['id'];
				$omInvoice->omnt_serial_number = $resultArray['Result']['Invoice']['serial_number'];
				$omInvoice->omnt_document_type = $invoiceSerial[1];
				$omInvoice->save();
			} else {
				Configuration::updateValue('OMNTORDEREXP_ORDER_MESSAGE', 'Ismeretlen hiba miatt nem sikerült létrehozni a rendeléshez a számlát. Ha szükséges, hozd létre azt kézzel a számlázóban.');
			}
		} else {
			Configuration::updateValue('OMNTORDEREXP_ORDER_MESSAGE', 'Ezen rendeléshez nem lett létrehozva bizonylat (számla, díjbekérő, stb), mivel nem lett beállítva számlatömb ehhez a devizához. Ha szükséges, hozd létre kézzel a számlázóban.');
		}
	}

	public function hookAdminOrder($params) {
		if (strlen(Configuration::get('OMNTORDEREXP_ORDER_MESSAGE'))>0) {
			global $smarty;
			$smarty->assign('omntorderexpOrderMessage', Configuration::get('OMNTORDEREXP_ORDER_MESSAGE'));
			Configuration::updateValue('OMNTORDEREXP_ORDER_MESSAGE', '');
			return $this->display(__FILE__, 'tpl/message.tpl');
		}
		return $this->_html;
	}

	private function calculateNet($value, $tax) {
		return ($value / (($tax/100)+1));
	}

	private function calculateGross($value, $tax) {
		return ($value * (($tax/100)+1));
	}
	
	public function hookActionOrderStatusPostUpdate($params) {
		if ($params['newOrderStatus']->id == Configuration::get('OMNTORDEREXP_EXPSTATUS')) {
			$this->CreateInvoice(array('id_order' => $params['id_order']));
		}
	}
}
?>